import{a as u,f as h,s as a}from"../chunks/mUIW7j_P.js";import{i as g}from"../chunks/DhfylSVL.js";import{p as l,i as v,t as d,j as _,s as x,k as e,l as o}from"../chunks/Cgca9gbR.js";import{s as k,p}from"../chunks/BcswWx0s.js";const $={get error(){return p.error},get status(){return p.status}};k.updated.check;const i=$;var b=h("<h1> </h1> <p> </p>",1);function y(m,f){l(f,!1),g();var r=b(),t=v(r),n=e(t,!0);o(t);var s=x(t,2),c=e(s,!0);o(s),d(()=>{a(n,i.status),a(c,i.error?.message)}),u(m,r),_()}export{y as component};
